@@ -1026,7 +1026,11 @@ private
   end
 
   def from_width
-    [(buffer.content_width.to_f * 0.2).to_i, MIN_FROM_WIDTH].max
+    if buffer
+      [(buffer.content_width.to_f * 0.2).to_i, MIN_FROM_WIDTH].max
+    else
+      MIN_FROM_WIDTH # not sure why the buffer is gone
+    end
   end
 
   def initialize_threads
